"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCollectionInner = exports.useCollectionState = exports.useCollection = exports.CollectionContext = void 0;
const react_1 = __importDefault(require("react"));
const __1 = require("..");
const Context = react_1.default.createContext({});
function CollectionContext(props = {}) {
    const { collection } = props;
    const context = react_1.default.useMemo(() => {
        return { collection };
    }, [collection]);
    return react_1.default.createElement(Context.Provider, { value: context }, props.children);
}
exports.CollectionContext = CollectionContext;
function useCollection(opts = {}) {
    const state = useCollectionState(opts);
    const res = state.collection || null;
    return res;
}
exports.useCollection = useCollection;
function useCollectionState(opts = {}) {
    var _a;
    // typeof opts extends { state: true } ?
    // AsyncCollectionState :
    // MaybeCollection {
    const context = (0, __1.useWorkspace)();
    if (!context)
        throw new Error('May not use useCollection hook outside of a WorkspaceProvider');
    const { workspace, defaultCollection } = context;
    const currentContext = react_1.default.useContext(Context);
    const collectionName = opts.collection || ((_a = currentContext.collection) === null || _a === void 0 ? void 0 : _a.name) || defaultCollection || undefined;
    const state = useCollectionInner(workspace, collectionName, opts);
    return state;
}
exports.useCollectionState = useCollectionState;
function useCollectionInner(workspace, collectionName, opts = {}) {
    const { liveUpdates } = opts;
    const [updateCounter, setUpdateCounter] = react_1.default.useState(0);
    const [openCounter, setOpenCounter] = react_1.default.useState(0);
    let state = (0, __1.useAsync)(async (workspace, collectionName) => {
        if (!workspace || !collectionName)
            return null;
        try {
            const collection = await workspace.openCollection(collectionName);
            return collection;
        }
        catch (err) {
            workspace.on('collection-open', collection => {
                if (collection.name !== collectionName)
                    return;
                setOpenCounter(i => i + 1);
            });
            throw err;
        }
    }, [workspace, collectionName, openCounter]);
    // console.log('useCollectionInner', state)
    const collection = state.data;
    const cstate = state;
    cstate.collection = collection;
    react_1.default.useEffect(() => {
        if (!collection || !liveUpdates)
            return;
        collection.pullLiveUpdates();
    }, [liveUpdates, collection]);
    react_1.default.useEffect(() => {
        if (!collection)
            return;
        const onevent = () => {
            setUpdateCounter(i => i + 1);
        };
        collection.on('open', onevent);
        collection.on('update', onevent);
        return () => {
            collection.removeListener('open', onevent);
            collection.removeListener('update', onevent);
        };
    }, [collection]);
    cstate.updated = updateCounter;
    return cstate;
    // if (opts.state) {
    //   return cstate as any
    // } else {
    //   return (cstate.collection || null) as any
    // }
}
exports.useCollectionInner = useCollectionInner;
//# sourceMappingURL=use-collection.js.map