"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onchange = exports.save = exports.load = exports.DEFAULT_CONFIG = exports.DEFAULT_KEY = void 0;
const react_1 = __importDefault(require("react"));
exports.DEFAULT_KEY = 'arsonar.config';
exports.DEFAULT_CONFIG = {
    url: 'http://localhost:9191/api/v1/default'
};
const config = {};
const listeners = {};
function load(key, defaultValue) {
    if (!key)
        throw new Error('Key is required');
    if (config[key] !== undefined)
        return config[key];
    const data = localStorage.getItem(key);
    if (data) {
        try {
            config[key] = JSON.parse(data);
        }
        catch (err) { }
    }
    if (config[key] === undefined) {
        if (typeof defaultValue === 'function') {
            defaultValue = defaultValue();
        }
        config[key] = { ...exports.DEFAULT_CONFIG, ...defaultValue };
    }
    return config[key];
}
exports.load = load;
function save(key, data) {
    if (!key)
        throw new Error('Key is required');
    localStorage.setItem(key, JSON.stringify(data));
    config[key] = data;
    if (listeners[key])
        listeners[key].forEach(fn => fn(data));
}
exports.save = save;
function onchange(fn, key = exports.DEFAULT_KEY) {
    listeners[key] = listeners[key] || [];
    listeners[key].push(fn);
    return () => { listeners[key] = listeners[key].filter(l => l !== fn); };
}
exports.onchange = onchange;
function useConfig(key = exports.DEFAULT_KEY, defaultValue = {}) {
    const [state, setState] = react_1.default.useState(() => load(key, defaultValue));
    react_1.default.useEffect(() => {
        return onchange(data => {
            setState(data);
        });
    }, [key]);
    function setConfig(nextData) {
        nextData = { ...state, ...nextData };
        save(key, nextData);
    }
    return [state, setConfig];
}
exports.default = useConfig;
//# sourceMappingURL=use-config.js.map