import React from 'react';
import { useAsync, useWorkspace } from '..';
const Context = React.createContext({});
export function CollectionContext(props = {}) {
    const { collection } = props;
    const context = React.useMemo(() => {
        return { collection };
    }, [collection]);
    return React.createElement(Context.Provider, { value: context }, props.children);
}
export function useCollection(opts = {}) {
    const state = useCollectionState(opts);
    const res = state.collection || null;
    return res;
}
export function useCollectionState(opts = {}) {
    var _a;
    // typeof opts extends { state: true } ?
    // AsyncCollectionState :
    // MaybeCollection {
    const context = useWorkspace();
    if (!context)
        throw new Error('May not use useCollection hook outside of a WorkspaceProvider');
    const { workspace, defaultCollection } = context;
    const currentContext = React.useContext(Context);
    const collectionName = opts.collection || ((_a = currentContext.collection) === null || _a === void 0 ? void 0 : _a.name) || defaultCollection || undefined;
    const state = useCollectionInner(workspace, collectionName, opts);
    return state;
}
export function useCollectionInner(workspace, collectionName, opts = {}) {
    const { liveUpdates } = opts;
    const [updateCounter, setUpdateCounter] = React.useState(0);
    const [openCounter, setOpenCounter] = React.useState(0);
    let state = useAsync(async (workspace, collectionName) => {
        if (!workspace || !collectionName)
            return null;
        try {
            const collection = await workspace.openCollection(collectionName);
            return collection;
        }
        catch (err) {
            workspace.on('collection-open', collection => {
                if (collection.name !== collectionName)
                    return;
                setOpenCounter(i => i + 1);
            });
            throw err;
        }
    }, [workspace, collectionName, openCounter]);
    // console.log('useCollectionInner', state)
    const collection = state.data;
    const cstate = state;
    cstate.collection = collection;
    React.useEffect(() => {
        if (!collection || !liveUpdates)
            return;
        collection.pullLiveUpdates();
    }, [liveUpdates, collection]);
    React.useEffect(() => {
        if (!collection)
            return;
        const onevent = () => {
            setUpdateCounter(i => i + 1);
        };
        collection.on('open', onevent);
        collection.on('update', onevent);
        return () => {
            collection.removeListener('open', onevent);
            collection.removeListener('update', onevent);
        };
    }, [collection]);
    cstate.updated = updateCounter;
    return cstate;
    // if (opts.state) {
    //   return cstate as any
    // } else {
    //   return (cstate.collection || null) as any
    // }
}
//# sourceMappingURL=use-collection.js.map